/******************************************************************************
*                                                                             *
*   E a s y C h a r                                                           *
*                                                                             *
*   by Stefan Heule                                                           *
*   member of boolsoft (www.boolsoft.org)                                     *
*                                                                             *
*   Source Code                                                               *
*                                                                             *
*******************************************************************************

EasyChar - Replacement of the standard char menu
Copyright (C) 2007-2010  Stefan Heule

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/



// ----------------------------------------------------------------- //
// --- header files
// ----------------------------------------------------------------- //
#include "header.h"




// ----------------------------------------------------------------- //
// save the file for the first time
// ----------------------------------------------------------------- //
// ----------------------------------------------------------------- //
void initBackup(void)
{
	// our charlist: 6 tabs with each 6 rows and 15 columns
	unsigned char char_list[24][15] = {
		// favorites
		{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: favorites
		{28,29,30,31,150,152,176,159,153,154,155,156,157,158,168},     // favorites 1
		{172,185,178,179,180,215,247,177,14,15,16,17,18,19,20},        // favorites 2
		{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: often used
		{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // often used 1
		{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // often used 2
		
		// math
		{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: math
		{28,29,30,31,150,152,176,159,153,154,155,156,157,158,168},     // math 1
		{172,185,178,179,180,215,247,177,0 ,0 ,0 ,0 ,0 ,0 ,0 },        // math 2
		{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: greek
		{128,129,130,131,132,133,134,135,136,137,181,138,139,140,141}, // greek 1
		{142,143,144,145,146,147,148,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0},          // greek 2
		
		// special
		{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: special
		{14,15,16,17,18,19,20,21,22,23,24,25,26,27,127},               // special
		{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: punctuation
		{33,'"','#','$','&','\'','?','@','_','`','|','~',160,161,162}, // punctuation 1
		{163,164,165,166,167,169,174,182,191,171,187,0 ,0 ,0 ,0 },     // punctuation 2
		{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // -- unused --
		
		// international
		{192,224,193,225,194,226,195,227,196,228,197,229,198,230,0 },  // A
		{200,232,201,233,202,234,203,235,0 ,0 ,0 ,0 ,0 ,0 ,0 },        // E
		{204,236,205,237,206,238,207,239,0 ,0 ,0 ,0 ,0 ,0 ,0 },        // I
		{210,242,211,243,212,244,213,145,214,246,216,248,186,0 ,0 },   // O
		{217,249,218,250,219,251,220,252,0 ,0 ,0 ,0 ,0 ,0 ,0 },        // U
		{199,231,208,240,209,241,223,222,254,221,253,255,0 ,0 ,0 },    // other
	};
	
	// fill the struct with data
	memcpy(pref.data, char_list, sizeof(char_list));
	pref.posx = pref.posy = pref.tab = 0;
	
	saveBackup();
}



// ----------------------------------------------------------------- //
// loads the configuration from a file
// ----------------------------------------------------------------- //
// local:
//     backup = config structure
//     file   = file structure
//     chcksm = file checksum
//     backup_file_magic = file magic
// return:
//     TRUE, or, in case of an error, FALSE
// note:
//     If there's no config file, a new one with default values will
//     be created
// ----------------------------------------------------------------- //
short loadBackup(void)
{
	OPTIONS backup;
	FILES file;
	unsigned char chcksm;
	unsigned char backup_file_magic[strlen(OPTIONS_FILE_MAGIC)+1];
	
	// unarchive the file
	EM_moveSymFromExtMem(SYMSTR(OPTIONS_FILE_NAME), HS_NULL);
	
	// open the backup file
	if (FOpen(OPTIONS_FILE_NAME, &file, FM_READ, "lst") != FS_OK)
	{
		// no file found
		//ST_helpMsg(MSG_ERROR_FILE_NOT_FOUND);
		initBackup();
		return TRUE;
	}
	
	// read the file magic
	backup_file_magic[strlen(OPTIONS_FILE_MAGIC)] = 0;
	if (FRead((void*)backup_file_magic, strlen(OPTIONS_FILE_MAGIC), &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_READ_FROM_FILE);
		return FALSE;
	}
	
	// check file version
	if (strcmp(OPTIONS_FILE_MAGIC, backup_file_magic) != 0)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_OUTDATED_FILE);
		return FALSE;
	}
	
	// read the file content
	if (FRead((void*)&backup, sizeof(OPTIONS), &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_READ_FROM_FILE);
		return FALSE;
	}
	
	// read the checksum
	if (FRead((void*)&chcksm, 1, &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_READ_FROM_FILE);
		return FALSE;
	}
	
	// check checksum
	if (checksum((void*)&backup, sizeof(OPTIONS)) != chcksm)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_CORRUPTED_FILE);
		return FALSE;
	}
	
	// close the file
	FClose(&file);
	
	pref = backup;
	
	// archive the file
	EM_moveSymToExtMem(SYMSTR(OPTIONS_FILE_NAME), HS_NULL);
	
	return TRUE;
}


// ----------------------------------------------------------------- //
// calculates a simple 1-byte-checksum of the provided data
// ----------------------------------------------------------------- //
// local:
//     md5 = MD5 structure
//     res = 16 byte MD5 checksum
// return:
//     first byte of MD5 checksum
// note:
//     only one byte of the 16-byte MD5 checksum will be used
// ----------------------------------------------------------------- //
unsigned char checksum(void* data, unsigned short bytes)
{
	unsigned char res[16];
	MD5_CTX md5;
	
	// get MD5 hash of the data
	MD5Init(&md5);
	MD5Update(&md5, data, bytes);
	MD5Final(res, &md5);
	
	// we just use the first byte of the 16 byte long MD5 hash
	return res[0];
}


// ----------------------------------------------------------------- //
// saves the configuration ot a file
// ----------------------------------------------------------------- //
// local:
//     file = file structure
// return:
//     TRUE, or, in case of an error, FALSE
// note:
//     Any old file will be overwritten
// ----------------------------------------------------------------- //
short saveBackup(void)
{
	FILES file;
	
	// unarchive the file
	EM_moveSymFromExtMem(SYMSTR(OPTIONS_FILE_NAME), HS_NULL);
	
	// open the backup file
	if (FOpen(OPTIONS_FILE_NAME, &file, FM_WRITE, "lst") != FS_OK)
	{
		ST_helpMsg(MSG_ERROR_CREATE_FILE);
		return FALSE;
	}
	
	// write the file magic
	if (FWrite((void*)OPTIONS_FILE_MAGIC, strlen(OPTIONS_FILE_MAGIC), &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_WRITE_TO_FILE);
		return FALSE;
	}
	
	// write backup
	if (FWrite((void*)&pref, sizeof(OPTIONS), &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_WRITE_TO_FILE);
		return FALSE;
	}
	
	// calculate checksum
	unsigned char chcksm = checksum((void*)&pref, sizeof(OPTIONS));
	
	// write the checksum
	if (FWrite((void*)&chcksm, 1, &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_WRITE_TO_FILE);
		return FALSE;
	}
	
	FClose(&file);
	
	// archive the file
	EM_moveSymToExtMem(SYMSTR(OPTIONS_FILE_NAME), HS_NULL);
	
	return TRUE;
}























