/******************************************************************************
*                                                                             *
*   E a s y C h a r                                                           *
*                                                                             *
*   by Stefan Heule                                                           *
*   member of boolsoft (www.boolsoft.org)                                     *
*                                                                             *
*   Source Code                                                               *
*                                                                             *
*******************************************************************************

EasyChar - Replacement of the standard char menu
Copyright (C) 2007-2010  Stefan Heule

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/



// ----------------------------------------------------------------- //
// --- header files
// ----------------------------------------------------------------- //
#include "header.h"



// ----------------------------------------------------------------- //
// --- globals
// ----------------------------------------------------------------- //
OPTIONS pref;                // the preferences
void* screen;                // backbuffer (NOT full screen size!)





// ----------------------------------------------------------------- //
// --- main
// ----------------------------------------------------------------- //
void _main(void)
{
	short i,j;                    // temp vars
	short key;                    // user input
	unsigned long len;            // length of the clipboard-content
	unsigned short clipboard = 0; // internal clipboard
	HANDLE h;                     // clipboard handle
	unsigned char buffer[10];     // temp buffer
	EVENT ev;                     // temp event
	
	
	// allocate some space for our virtual scree (backbuffer)
	if ((screen = malloc(2*95*18*8)) == NULL) {
		ST_helpMsg(MSG_ERROR_MEMORY);
		return;
	}
	
	// load the charlist (loadBackup will output an error msg)
	if (loadBackup() == FALSE)
		return;
	
	// if there is one single char in the clipboard, put it into our internal clipboard
	if (CB_fetchTEXT(&h, &len) && len == 1)
		clipboard = *((const char*)HeapDeref(h));
	
	// backup orginal screen
	for (i = 0; i < 95; i++)
		memcpy(screen+95*18*8+i*18, LCD_MEM+(i+OFFSET_Y)*30+OFFSET_X, 18);
	
	// set the graphic port to our virtual screen for backbuffering
	PortSet(screen, 143, 94);
	
	// main loop
	while (1)
	{
		// check if the cursor is at a valid position
		i = 0;
		while (pref.data[(pref.tab*6+pref.posy)*15+pref.posx] == 0)
		{
			if (i > 15)
			{
				add(&pref.posy, 5);
				pref.posx = -1;
			}
			i++;
			add(&pref.posx, 14);
		}
		
		// main border and (white) background
		memset(screen, 0, 144*95);
		DrawRect(0, 2, 142, 94, 0, 2);
		
		// titlebar and title
		FontSetSys(F_4x6);
		DrawRect(2, 0, 140, 11, 1, 1);
		DrawStr(3, 4, MSG_ABOUT_PROGRAM_NAME" "MSG_ABOUT_VERSION, A_XOR);
		DrawPix(1, 1, 1);
		DrawPix(141, 1, 1);
		
		// tabs
		DrawRect(3, 13, 139, 21, 0, 1);
		DrawStr(4, 15, MSG_TAB_NAMES, A_XOR);
		DrawLine(36, 13, 36, 21, 1);
		DrawLine(59, 13, 59, 21, 1);
		DrawLine(90, 13, 90, 21, 1);
		
		// main tab
		DrawRect(3, 21, 139, 83, 0, 1);
		
		// status line
		DrawRect(3, 83, 139, 91, 0, 1);
		DrawStr(138-DrawStrWidth("F1:"MSG_HELP, F_4x6), 85, "F1:"MSG_HELP, A_XOR);
		if (pref.data[(pref.tab*6+pref.posy)*15+pref.posx] != 0) {
			sprintf(buffer, "0x%02X, %i", pref.data[(pref.tab*6+pref.posy)*15+pref.posx], pref.data[(pref.tab*6+pref.posy)*15+pref.posx]);
			DrawStr(4, 85, buffer, A_XOR);
		}
		
		// draw all char's
		FontSetSys(F_6x8);
		for (i = 0; i < 15; i++)
			for (j = 0; j < 6; j++)
				if (pref.data[(pref.tab*6+j)*15+i] != 0)
					DrawChar(i*9+5, j*10+24, pref.data[(pref.tab*6+j)*15+i], A_XOR);
		
		// draw the selector
		DrawRect(pref.posx*9+5, pref.posy*10+23, pref.posx*9+11, pref.posy*10+31, 1, 3);
		
		// tab specific things
		FontSetSys(F_4x6);
		if (pref.tab == 0) {
			DrawLine(4, 21, 35, 21, 0);
			DrawStr(5, 56, MSG_TAB_LASTUSED":", 1);
			DrawStr(5, 26, MSG_TAB_FAVORITES":", 1);
		}
		else if (pref.tab == 1) {
			DrawLine(37, 21, 58, 21, 0);
			DrawStr(5, 26, MSG_TAB_MATH":", 1);
			DrawStr(5, 56, MSG_TAB_GREEK":", 1);
		}
		else if (pref.tab == 2) {
			DrawLine(60, 21, 89, 21, 0);
			DrawStr(5, 26, MSG_TAB_SPECIAL":", 1);
			DrawStr(5, 46, MSG_TAB_PUNCTUATION":", 1);
		}
		else if (pref.tab == 3) {
			DrawLine(91, 21, 138, 21, 0);
		}
		
		// copy everything to the screen
		switchBackbuffer();
		
		// check for a keypress
		key = getKey();
		
		// cursor right
		if (key == KEY_RIGHT) {
			add(&pref.posx, 14);
			i = 0;
			while (pref.data[(pref.tab*6+pref.posy)*15+pref.posx] == 0) {
				if (i > 15) {
					add(&pref.posy, 5);
					pref.posx = -1;
				}
				i++;
				add(&pref.posx, 14);
			}
		}
		
		// cursor left
		else if (key == KEY_LEFT) {
			sub(&pref.posx, 14);
			i = 0;
			while (pref.data[(pref.tab*6+pref.posy)*15+pref.posx] == 0) {
				if (i > 15) {
					sub(&pref.posy, 5);
					pref.posx = 15;
				}
				i++;
				sub(&pref.posx, 14);
			}
		}
		
		// cursor up
		else if (key == KEY_UP)
		{
			sub(&pref.posy, 5);
			i = 0;
			while (pref.data[(pref.tab*6+pref.posy)*15+pref.posx] == 0) {
				if (i > 15) {
					sub(&pref.posx, 14);
					pref.posy = 6;
				}
				i++;
				sub(&pref.posy, 5);
			}
		}
		
		// cursor down
		else if (key == KEY_DOWN) {
			add(&pref.posy, 5);
			i = 0;
			while (pref.data[(pref.tab*6+pref.posy)*15+pref.posx] == 0) {
				if (i > 6) {
					add(&pref.posx, 14);
					pref.posy = -1;
				}
				i++;
				add(&pref.posy, 5);
			}
		}
		
		// pasting the char without leaving
		else if (key == KEY_DIAMOND+KEY_ENTER || key == KEY_ENTER+4096)
		{
			// restore the graphic port
			PortRestore();
			
			// restore screen
			for (i = 0; i < 95; i++)
				memcpy(LCD_MEM+(i+OFFSET_Y)*30+OFFSET_X, screen+95*18*8+i*18, 18);
			
			// init the event
			EVENT ev;
			ev.Type = CM_PASTE_STRING;
			buffer[0] = pref.data[(pref.tab*6+pref.posy)*15+pref.posx];
			buffer[1] = 0;
			ev.extra.pasteText = buffer;
			
			// last used
			for (i = 0; i < 15; i++) {
				for (j = 4; j < 6; j++) {
					
					// if the char already was in the often used list, move it to the begining
					if (pref.data[j*15+i] == pref.data[(pref.tab*6+pref.posy)*15+pref.posx]) {
						memmove(&pref.data[4*15+1], &pref.data[4*15+0], (j-4)*15+i);
						pref.data[4*15+0] = pref.data[(pref.tab*6+pref.posy)*15+pref.posx];
						i = j = 120; // out of both loops
					}
					
					// if no entry is found
					if (j == 5 && i == 14) {
						memmove(&pref.data[4*15+1], &pref.data[4*15+0], 29);
						pref.data[4*15+0] = pref.data[(pref.tab*6+pref.posy)*15+pref.posx];
					}
				}
			}
			
			// send the event
			EV_sendEvent(AP_CURRENT, &ev);
			
			// backup orginal screen
			for (i = 0; i < 95; i++)
				memcpy(screen+95*18*8+i*18, LCD_MEM+(i+OFFSET_Y)*30+OFFSET_X, 18);
			
			// set the graphic port to our virtual screen for backbuffering
			PortSet(screen, 143, 94);
		}
		
		// next tab
		else if (key == KEY_RIGHT+KEY_DIAMOND || key == KEY_APPS) {
			add(&pref.tab, 3);
			pref.posx = pref.posy = 0;
		}
		
		// previous tab
		else if (key == KEY_LEFT+KEY_DIAMOND) {
			sub(&pref.tab, 3);
			pref.posx = pref.posy = 0;
		}
		
		// exit
		else if (key == KEY_ESC) {
			// restore the graphic port
			PortRestore();
			
			// restore screen
			for (i = 0; i < 95; i++)
				memcpy(LCD_MEM+(i+OFFSET_Y)*30+OFFSET_X, screen+95*18*8+i*18, 18);
			
			break;
		}
		
		// help
		else if (key == KEY_F1) {
			// white background
			DrawRect(4, 14, 138, 90, 1, 0);
			
			//title
			DrawStr(124, 4, MSG_HELP, A_XOR);
			
			DrawStr(6, 17+0*8, "ENTER", A_XOR);
			DrawStr(6, 17+1*8, "\x7F"" + ENTER", A_XOR);
			DrawStr(6, 17+2*8, "\x7F"" + \x11/\x12", A_XOR);
			DrawStr(6, 17+3*8, "APPS", A_XOR);
			DrawStr(6, 17+4*8, "2nd+"MSG_HELP_ARROWS, A_XOR);
			DrawStr(6, 17+5*8, "\x7F"" + C/V", A_XOR);
			DrawStr(6, 17+6*8, "F2", A_XOR);
			DrawStr(6, 17+7*8, "F3", A_XOR);
			DrawStr(6, 17+8*8, "F4", A_XOR);
			
			DrawStr(47, 17+0*8, MSG_HELP_PASTE_CHAR, A_XOR);
			DrawStr(47, 17+1*8, MSG_HELP_PASTE_DONT_LEAVE, A_XOR);
			DrawStr(47, 17+2*8, MSG_HELP_SWITCH_TABS, A_XOR);
			DrawStr(47, 17+3*8, MSG_HELP_SWITCH_TABS, A_XOR);
			DrawStr(47, 17+4*8, MSG_HELP_JUMP, A_XOR);
			DrawStr(47, 17+5*8, MSG_HELP_COPY_PASTE, A_XOR);
			DrawStr(47, 17+6*8, MSG_HELP_OPEN_OLD_MENU, A_XOR);
			DrawStr(47, 17+7*8, MSG_HELP_OPEN_SET_CHAR, A_XOR);
			DrawStr(47, 17+8*8, MSG_ABOUT, A_XOR);
			
			// copy everything to the screen
			switchBackbuffer();
			
			getKey();
		}
		
		// about
		else if (key == KEY_F4) {
			// white background
			DrawRect(4, 14, 138, 90, 1, 0);
			
			//title
			DrawStr(119, 4, MSG_ABOUT, A_XOR);
			
			FontSetSys(F_6x8);
			DrawStr(71-DrawStrWidth(MSG_ABOUT_PROGRAM_NAME, F_6x8)/2,                      25, MSG_ABOUT_PROGRAM_NAME, A_XOR);
			FontSetSys(F_4x6);
			DrawStr(71-DrawStrWidth("v"MSG_ABOUT_VERSION, F_4x6)/2, 35, "v"MSG_ABOUT_VERSION, A_XOR);
			DrawStr(71-DrawStrWidth(MSG_ABOUT_BY_AUTHOR, F_4x6)/2,                         50, MSG_ABOUT_BY_AUTHOR, A_XOR);
			DrawStr(71-DrawStrWidth(MSG_ABOUT_BOOLSOFT_MEMBER, F_4x6)/2,                   58, MSG_ABOUT_BOOLSOFT_MEMBER, A_XOR);
			DrawStr(71-DrawStrWidth(MSG_ABOUT_BOOLSOFT_URL, F_4x6)/2,                      73, MSG_ABOUT_BOOLSOFT_URL, A_XOR);
			
			// copy everything to the screen
			switchBackbuffer();
			
			getKey();
		}
		
		// old menu
		else if (key == KEY_F2) {
			// restore screen
			for (i = 0; i < 95; i++)
				memcpy(LCD_MEM+(i+OFFSET_Y)*30+OFFSET_X, screen+95*18*8+i*18, 18);
			
			PortRestore();
			
			// send the event
			/*ev.Type = CM_KEYPRESS;
			ev.extra.Key.Code = KEY_CHAR;
			EV_sendEvent(AP_CURRENT, &ev);*/
			ev.Type = CM_KEYPRESS;
			ev.extra.Key.Code = KEY_CHAR;
			EV_defaultHandler(&ev);
			break;
		}
		
		// new char
		else if (key == KEY_F3) {
			short char_num = 0;
			short cursor = 0;
			
			// set buffer to 0
			memset(buffer, 0, 4);
			
			do
			{
				// draw everything
				
				// draw dialog
				DrawDialog(27, 29, 107, 64, MSG_MISC_FIND_CHAR);
				
				// input field
				DrawRect(85, 40, 102, 48, 0, 1);
				
				// draw string
				FontSetSys(F_4x6);
				DrawStr(87, 42, buffer, A_XOR);
				
				// string "enter number"
				DrawStr(31, 42, MSG_MISC_ENTER_NUMBER":", A_XOR);
				
				// string "preview"
				DrawStr(31, 53, MSG_MISC_PREVIEW":", A_XOR);
				
				// draw preview
				FontSetSys(F_6x8);
				if (atoi(buffer) > 0)
					DrawChar(95, 52, atoi(buffer), 1);
				
				// cursor
				strcpy(&buffer[5], buffer);
				buffer[5+cursor] = 0;
				j = DrawStrWidth(&buffer[5], F_4x6);
				DrawLine(87+j, 41, 87+j, 47, 1);
				
				switchBackbuffer();
				
				// get a input
				key = getKey();
				
				if ((key >= '0' && key <= '9') && char_num < 3)
				{
					for (i = char_num; i > cursor; i--)
						buffer[i] = buffer[i-1];
					buffer[cursor] = key;
					char_num++;
					buffer[char_num] = 0;
					cursor++;
					
					// check for limit
					if (atoi(buffer) > 255)
						sprintf(buffer, "%i", 255);
				}
				
				else if ((key == KEY_BACKSPACE) && (char_num > 0) && (cursor > 0)) {
					for (i = cursor-1; i < char_num; i++)
						buffer[i] = buffer[i+1];
					char_num--;
					buffer[char_num] = 0;
					cursor--;
				}
				
				else if (key == KEY_ENTER) {
					i = atoi(buffer);
					break;
				}
				else if (key == KEY_ESC) {
					i = 0;
					break;
				}
				else if (key == KEY_RIGHT && cursor < char_num) {
					cursor++;
				}
				else if (key == KEY_LEFT && cursor > 0) {
					cursor--;
				}
				else if (key == KEY_UP) {
					strcpy(&buffer[5], buffer);
					buffer[5+strlen(buffer)] = 0;
					i = atoi(&buffer[5]);
					if (i == 255) i--;
					sprintf(buffer, "%i", i+1);
					cursor = strlen(buffer);
				}
				else if (key == KEY_DOWN) {
					strcpy(&buffer[5], buffer);
					buffer[5+strlen(buffer)] = 0;
					i = atoi(&buffer[5]);
					if (i == 0) i++;
					sprintf(buffer, "%i", i-1);
					cursor = strlen(buffer);
				}
				char_num = strlen(buffer);
			} while (TRUE);
			
			// set new char
			if (i)
			{
				if (pref.posy > 0 && pref.posy < 3 && pref.tab == 0)
				{
					pref.data[(pref.tab*6+pref.posy)*15+pref.posx] = i;
				}
				else
				{
					clipboard = i;
					ST_helpMsg(MSG_WARNING_NO_REPLACE_CLIP);
				}
			}
			
			key = 0;
		}
		
		// store the char in the internal clipboard
		else if (key == KEY_DIAMOND+'C')
			clipboard = pref.data[(pref.tab*6+pref.posy)*15+pref.posx];
		
		// replace the actual char (if it is a favorite)
		else if (key == KEY_DIAMOND+'V' && clipboard) {
			if (pref.posy > 0 && pref.posy < 3 && pref.tab == 0)
				pref.data[(pref.tab*6+pref.posy)*15+pref.posx] = clipboard;
			else
				ST_helpMsg(MSG_WARNING_NO_REPLACE);
		}
		
		// end of line
		else if (key == 4096+KEY_RIGHT)
			while (pref.data[(pref.tab*6+pref.posy)*15+pref.posx+1] && pref.posx < 14)
				pref.posx++;
		
		// beginning of line
		else if (key == 4096+KEY_LEFT)
			while (pref.data[(pref.tab*6+pref.posy)*15+pref.posx-1] && pref.posx > 0)
				pref.posx--;
		
		// end of column
		else if (key == 4096+KEY_DOWN) {
			for (i = 5; i > pref.posy; i--)
				if (pref.data[(pref.tab*6+i)*15+pref.posx]) {
					pref.posy = i;
					break;
				}
		}
		
		// beginning of column
		else if (key == 4096+KEY_UP) {
			for (i = 0; i < pref.posy; i++)
				if (pref.data[(pref.tab*6+i)*15+pref.posx]) {
					pref.posy = i;
					break;
				}
		}
		
		// paste the character
		else if (key == KEY_ENTER) {
			// last used
			for (i = 0; i < 15; i++)
				for (j = 4; j < 6; j++) {
					// if the char already was in the often used list, move it to the begining
					if (pref.data[j*15+i] == pref.data[(pref.tab*6+pref.posy)*15+pref.posx]) {
						memmove(&pref.data[4*15+1], &pref.data[4*15+0], (j-4)*15+i);
						pref.data[4*15+0] = pref.data[(pref.tab*6+pref.posy)*15+pref.posx];
						i = j = 120; // out of both loops
					}
					
					// if no entry is found
					if (j == 5 && i == 14) {
						memmove(&pref.data[4*15+1], &pref.data[4*15+0], 29);
						pref.data[4*15+0] = pref.data[(pref.tab*6+pref.posy)*15+pref.posx];
					}
				}
			
			// update the clipboard
			if (clipboard) {
				buffer[0] = clipboard;
				buffer[1] = 0;
				CB_replaceTEXT(buffer, 1, 0);
			}
			
			// restore screen
			for (i = 0; i < 95; i++)
				memcpy(LCD_MEM+(i+OFFSET_Y)*30+OFFSET_X, screen+95*18*8+i*18, 18);
			
			// restore the graphic port
			PortRestore();
			
			// set the event to <paste>
			ev.Type = CM_PASTE_STRING;
			buffer[0] = pref.data[(pref.tab*6+pref.posy)*15+pref.posx];
			buffer[1] = 0;
			ev.extra.pasteText = buffer;
			EV_sendEvent(AP_CURRENT, &ev);
			
			break;
		}
	}
	
	// store the internal clipboard in the TIOS clipboard
	if (clipboard && key != KEY_ENTER)
	{
		buffer[0] = clipboard;
		buffer[1] = 0;
		CB_replaceTEXT(buffer, 1, 0);
	}
	
	// free the screen buffer
	free(screen);
	
	// erase the status line
	ST_eraseHelp();
	
	saveBackup();
}





































